#!/usr/bin/env node

/* $filename: $ */
"use strict";

var _index = _interopRequireDefault(require("../index.js"));

var _optimist = _interopRequireDefault(require("optimist"));

var _debug = _interopRequireDefault(require("@norjs/debug"));

var _is = _interopRequireDefault(require("@norjs/is"));

var _fs = _interopRequireDefault(require("fs"));

var _q = _interopRequireDefault(require("q"));

var _rimraf = _interopRequireDefault(require("rimraf"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var argv = _optimist.default.boolean('v').argv;

if (argv.v) {
  _debug.default.setNodeENV('development');
} else {
  _debug.default.setNodeENV('production');
}

_debug.default.setProjectRoot(require('path').dirname(__dirname));

_q.default.longStackSupport = true;

var rimraf = _q.default.denodeify(_rimraf.default);

var pgrunner_config_file = require('path').resolve(process.env.HOME, '.nor-pgrunner.json');
/** Returns current config
 *
 */


function load_config() {
  var config = {};

  if (_fs.default.existsSync(pgrunner_config_file)) {
    config = JSON.parse(_fs.default.readFileSync(pgrunner_config_file, {
      'encoding': 'utf8'
    }));

    if (argv.v) {
      _debug.default.log('Loaded from ', pgrunner_config_file);
    }
  }

  if (!_is.default.array(config.servers)) {
    config.servers = [];
  }

  return config;
}
/** Save current config
 *
 * @param config
 */


function save_config(config) {
  _fs.default.writeFileSync(pgrunner_config_file, JSON.stringify(config, null, 2), {
    'encoding': 'utf8'
  });

  if (argv.v) {
    _debug.default.log('Saved to ', pgrunner_config_file);
  }
}
/** Strip argv
 *
 * @param a
 * @returns {any[]}
 */


function strip_argv(a) {
  var o = {};
  return Object.keys(a).filter(function (k) {
    return k !== '_';
  }).map(function (k) {
    o[k] = a[k];
  });
}
/** Strip keys
 *
 * @param old_opts
 */


function strip_server_opts(old_opts) {
  _debug.default.assert(old_opts).is('object');

  var opts = {};
  ['pgconfig', 'host', 'port', 'user', 'database'].forEach(function (key) {
    if (old_opts[key] !== undefined) {
      opts[key] = old_opts[key];
    }
  });
  return opts;
}
/** Returns server options
 *
 * @param opts
 * @returns {{database: *, port: (*|number), dbconfig: *, host: (*|string), user: *}}
 */


function get_server_opts(opts) {
  _debug.default.assert(opts).is('object');

  _debug.default.assert(opts.settings).is('object');

  return {
    'dbconfig': opts.dbconfig,
    'host': opts.settings.host,
    'port': opts.settings.port,
    'user': opts.settings.user,
    'database': opts.settings.database
  };
}
/**
 *
 * @param servers
 * @param opts
 * @returns {*|T}
 */


function search_server(servers, opts) {
  _debug.default.assert(servers).is('array');

  _debug.default.assert(opts).is('object');

  var instance_opts = servers.filter(function (server) {
    var server_opts = get_server_opts(server);
    return Object.keys(opts).map(function (key) {
      return opts[key] === server_opts[key];
    }).every(function (v) {
      return v === true;
    });
  }).shift();

  if (!instance_opts) {
    throw new Error('Could not find server!');
  }

  return instance_opts;
}

var _commands = {
  'create': function create(opts) {
    return (0, _index.default)(opts).then(function (instance) {
      // Save config
      var config = load_config();
      config.servers.push(instance);
      save_config(config); // Print to screen

      console.log(instance.pgconfig);
    });
  },
  'list': function list()
  /*opts*/
  {
    var config = load_config();

    if (argv.v) {
      _debug.default.log('config = ', config);
    }

    console.log("pgconfig\n--------\n" + config.servers.map(function (server) {
      return [server.pgconfig].join(' | ');
    }).join('\n'));
  },
  'destroy': function destroy(opts) {
    // Load config
    var config = load_config();

    if (argv.v) {
      _debug.default.log('config = ', config);
    }

    opts = strip_server_opts(opts);
    var instance_opts = search_server(config.servers, opts);

    if (argv.v) {
      _debug.default.log('instance_opts = ', instance_opts);
    }

    _debug.default.assert(instance_opts).is('object');

    instance_opts.MARKED_FOR_DELETE = true;
    var instance = new _index.default.Instance(instance_opts);

    _debug.default.assert(instance).is('object');

    if (argv.v) {
      _debug.default.log('Stopping...');
    }

    return instance.stop().then(function () {
      // Remove the temp directory
      _debug.default.assert(instance).is('object');

      _debug.default.assert(instance.env).is('object');

      _debug.default.assert(instance.env.PGDATA).is('string');

      return rimraf(instance.env.PGDATA);
    }).then(function () {
      // Save config without the record we stopped
      config.servers = config.servers.filter(function (s) {
        return s.MARKED_FOR_DELETE === undefined;
      });
      save_config(config);
      console.log('Destroyed successfully');
    });
  }
};

_q.default.fcall(function () {
  var steps = argv._.map(function (cmd) {
    if (_is.default.func(_commands[cmd])) {
      return _commands[cmd].bind(undefined, strip_argv(argv));
    } else {
      throw new Error('Unknown command: ' + cmd);
    }
  });

  if (steps.length < 1) {
    console.log(`USAGE: pgrunner OPT(s) ACTION(s)

where ACTION is:
  create  -- Create a server
  start   -- Start a server [not implemented]
  stop    -- Stop a server [not implemented]
  list    -- List servers
  destroy -- Delete a server

where OPT(s) are:
  -v                 -- Write more information
  --pgconfig=CONFIG  -- Search by pg config string
  --host=HOST        -- Match by host
  --port=PORT        -- Match by port
  --user=USER        -- Match by user
  --database=NAME    -- Match by database name
`);
  }

  return steps.reduce(_q.default.when, (0, _q.default)());
}).catch(function (err) {
  if (err.retval && err.stderr) {
    process.stderr.write(err.stderr + '\n');
    return;
  }

  process.stderr.write('Error: ' + err + '\n');

  if (err.stack) {
    _debug.default.error(err.stack);
  }
}).done();
/* EOF */