/* pgrunner library */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pgrunner = pgrunner;
exports.default = exports.PGRunnerInstance = void 0;

var _Async = require("./Async.js");

var _debug = _interopRequireDefault(require("@norjs/debug"));

var _is = _interopRequireDefault(require("@norjs/is"));

var _child_process = _interopRequireDefault(require("child_process"));

var _q = _interopRequireDefault(require("q"));

var _merge = _interopRequireDefault(require("merge"));

var _temp = _interopRequireDefault(require("temp"));

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _readOnlyError(name) { throw new Error("\"" + name + "\" is read-only"); }

var PG_PATHS = ['/usr/lib/postgresql/9.3/bin', '/usr/lib/postgresql/9.4/bin', '/usr/local/Cellar/postgresql/11.1/bin'];
/**
 * Function that returns promise of a temporary directory
 */

var mktempdir = _q.default.denodeify(_temp.default.mkdir);
/** Search command from PATH
 *
 * @param paths {string|array}
 * @param name
 * @returns {*|boolean}
 */


function command_exists(paths, name) {
  if (_is.default.string(paths)) {
    paths = paths.split(':');
  }

  _debug.default.assert(paths).is('array');

  return paths.some(function (dir) {
    return _fs.default.existsSync(_path.default.join(dir, name));
  });
}
/**
 *
 * @type {string}
 */


var PGCTL_TIMEOUT = "30";
/**
 *
 * @type {string}
 */

var PG_CTL = 'pg_ctl';
/**
 *
 * @type {*[]}
 */

var PG_CTL_START_OPTS = ["start", "-s", "-t", PGCTL_TIMEOUT, "-w", "-o", "-F -k /tmp"];
/**
 *
 * @type {*[]}
 */

var PG_CTL_STOP_OPTS = ["stop", "-s", "-t", PGCTL_TIMEOUT, "-w", "-m", "fast"];

if (!command_exists(process.env.PATH, 'pg_ctl')) {
  PG_PATHS.forEach(function (bin_dir) {
    if (_fs.default.existsSync(_path.default.join(bin_dir, 'pg_ctl'))) {
      process.env.PATH = bin_dir + ':' + process.env.PATH;
      PG_CTL = (_readOnlyError("PG_CTL"), _path.default.join(bin_dir, 'pg_ctl'));
    }
  });
} //debug.log('pg_ctl detected as ', PG_CTL);

/** Return promise of a spawned command
 *
 * @param command {string}
 * @param args {array.<string>}
 * @param options {object}
 * @returns {*}
 */


function spawnProcess(command, args) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$env = _ref.env,
      env = _ref$env === void 0 ? {} : _ref$env;

  return _Async.Async.Promise(function (resolve, reject) {
    //debug.log('command = ', command);
    //debug.log('args = ', args);
    //debug.log('options = ', options);
    // NOTE! If you set stdout to be captured instead of ignored (the postgres log is there),
    // pgctl start will fail to exit.
    var options = {
      env: (0, _merge.default)(process.env, env || {}),
      detached: true,
      stdio: ["ignore", "ignore", "pipe"]
    };
    var stderr = ''; // Run the process
    //debug.log('Executing command ', command, args);

    var proc = _child_process.default.spawn(command, args, options); // Handle exit


    proc.on('close', function (retval) {
      //debug.log('Command ', command, args, ' closed with ', retval);
      if (retval === 0) {
        resolve(retval);
      } else {
        reject({
          retval,
          stderr
        });
      }
    }); // Handle error

    proc.on('error', function (err) {
      reject(err);
    }); //proc.stdout.setEncoding('utf8');
    //proc.stdout.on('data', data => {
    //	process.stderr.write(data);
    //});

    proc.stderr.setEncoding('utf8');
    proc.stderr.on('data', function (data) {
      stderr += data;
    });
  });
}
/**
 *
 */


var PGRunnerInstance =
/*#__PURE__*/
function () {
  /** Constructor
   *
   * @param settings {object}
   * @param env {object}
   * @param pgconfig {string|undefined}
   */
  function PGRunnerInstance() {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        _ref2$settings = _ref2.settings,
        settings = _ref2$settings === void 0 ? {} : _ref2$settings,
        _ref2$env = _ref2.env,
        env = _ref2$env === void 0 ? {} : _ref2$env,
        _ref2$pgconfig = _ref2.pgconfig,
        pgconfig = _ref2$pgconfig === void 0 ? undefined : _ref2$pgconfig;

    _classCallCheck(this, PGRunnerInstance);

    this.settings = settings || {};
    this.env = env || {};
    this.pgconfig = pgconfig || undefined;
  }
  /** Stop the instance */


  _createClass(PGRunnerInstance, [{
    key: "stop",
    value: function stop() {
      //debug.log("Stopping PostgreSQL");
      //debug.log("env = ", this.env);
      return spawnProcess(PG_CTL, PG_CTL_STOP_OPTS, {
        "env": this.env
      }).then(function () {});
    }
  }]);

  return PGRunnerInstance;
}();
/** The Module
 *
 * @param opts
 * @returns {PromiseLike<PGRunnerInstance | never>}
 */


exports.PGRunnerInstance = PGRunnerInstance;

function pgrunner() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var pghost = opts.host || '127.0.0.1';
  var pgport = opts.port || 55432;
  var pguser = opts.user || process.env.USER;
  var pgdatabase = opts.database || pguser;
  var instance = new PGRunnerInstance(); // Create and start the database

  return mktempdir('nor-pgrunner-data-').then(function (tmpdir) {
    //debug.log('Created temp directory: ', tmpdir);
    instance.settings = {
      'host': pghost,
      'port': pgport,
      'user': pguser,
      'database': pgdatabase
    };
    instance.env = {
      "PGDATA": tmpdir,
      "PGHOST": pghost,
      "PGPORT": pgport,
      "PGUSER": pguser,
      "PGDATABASE": pgdatabase,
      "PGCLIENTENCODING": "UTF-8",
      "TZ": "UTC"
    };
    instance.pgconfig = "postgresql://" + pguser + "@" + pghost + ":" + pgport + "/" + pgdatabase; //debug.log('Instance ENV: ', instance.env);
    //debug.log('Instance pgconfig: ', instance.pgconfig);
  }).then(function () {
    //debug.log("Initializing PostgreSQL database");
    return spawnProcess(PG_CTL, ["init", "-s", "-t", PGCTL_TIMEOUT, "-w", "-o", "-N -U " + pguser], {
      "env": instance.env
    });
  }).then(function () {
    //debug.log("Starting PostgreSQL");
    return spawnProcess(PG_CTL, PG_CTL_START_OPTS, {
      "env": instance.env
    });
  }).then(function () {
    //debug.log("Creating database");
    return spawnProcess('createdb', ["-w", "-h", pghost, "-p", pgport, "-U", pguser, "-O", pguser, pgdatabase], {
      "env": instance.env
    });
  }).then(function () {
    return instance;
  });
}

pgrunner.Instance = PGRunnerInstance;
/** Automatically track and cleanup files at exit
 */

pgrunner.enableAutoClean = function () {
  _temp.default.track(); // FIXME: Does not shutdown servers when cleaning up files.

};

var _default = pgrunner;
/* EOF */

exports.default = _default;